﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BoardManager : MonoBehaviour
{
    public HexGrid Grid = null;

    [SerializeField]
    public int BoardWidth = 42;
    public int halfWidth = 0;

    [SerializeField]
    public int BoardHeight = 20;
    public int halfHeight = 0;

    [SerializeField]
    private GameObject Rose = null;

    public LevelInformation DefaultEditorLevel = null;

    void Start()
    {
        Grid = new HexGrid();
        Grid.SetGridSize(0.5f);

        if (DefaultEditorLevel != null)
        {
            GameState.Instance.NextLevelToPlay = DefaultEditorLevel;
        }

        // Even though we use Axial co-ordinates throughout the game, board creation itself will be done in Offset co-ordinates to match the 2D shape of the screen
        BoardWidth = GameState.Instance.NextLevelToPlay.BoardWidth;
        BoardHeight = GameState.Instance.NextLevelToPlay.BoardHeight;

        halfWidth = BoardWidth / 2;
        halfHeight = BoardHeight / 2;

        Camera.main.orthographicSize = GameState.Instance.NextLevelToPlay.OrthoCameraSize;

        for (int y = -halfHeight; y <= halfHeight; y++)
        {
            for (int x = -halfWidth; x <= halfWidth; x++)
            {
                Vector3 spawnPosition = Grid.WorldPositionFromAxialPosition(new AxialCoordinate(new OffsetCoordinate(x, y)));
                GameObject newRose = GameObject.Instantiate(Rose, spawnPosition, Quaternion.Euler(0.0f, Random.Range(0, 360.0f), 0.0f));
                Rose rose = newRose.GetComponent<Rose>();
                rose.FlowerColor = new Color(1.0f, Random.value, Random.value);
            }
        }
        
    }

    readonly LayerMask mask = 0x01;

    public Rose GetRoseAt(AxialCoordinate location)
    {
        Grid.QueryForCollidersAtAxialPosition(location, mask);
        if (Grid.GetQueriedColliderCount() > 0)
        {
            foreach (Collider c in Grid.IterateQueriedColliders())
            {
                return c.GetComponent<Rose>();
            }
        }
        return null;
    }

    // Create a hole at the provided location or return the hole that is already there
    public Rose CreateOrGetHole(AxialCoordinate holeLocation)
    {
        // Retrieve the rose at the passed location
        Rose rose = GetRoseAt(holeLocation);

        if (rose != null)
            rose.Alive = false;

        return rose;
    }

    // Attempts to fill a hole at the given location, fails silently if no hole is there
    public void FillHole(AxialCoordinate holeLocation)
    {
        Rose rose = GetRoseAt(holeLocation);
        if (rose != null)
            rose.Alive = true;
    }

    public bool IsHoleAt(AxialCoordinate holeLocation)
    {
        // Check if a hole is already there
        Rose rose = GetRoseAt(holeLocation);
        if (rose != null)
            return !rose.Alive;

        return true;
    }

    public bool WithinBounds(AxialCoordinate location)
    {
        OffsetCoordinate coord = new OffsetCoordinate(location);
        return (coord.col >= -halfWidth && coord.col <= halfWidth && coord.row >= -halfHeight && coord.row <= halfHeight);
    }
}
