﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;

public class ConveyorManager : MonoBehaviour
{
    public Transform ConveyorParent;
    public GameObject WonderGroButton = null;
    public List<GameObject> ActiveButtons = new List<GameObject>();

    public float ConveyorVelocity = -20.0f;
    public float PaddingBetweenButtons = 5.0f;

    // Start is called before the first frame update
    void Start()
    {
        
    }

    public Color AvailableButtonColor = new Color(0.2f, 0.42f, 0.15f);
    public Color ActiveButtonColor = new Color(0.63f, 0.78f, 0.18f);

    public UnityEvent PieceClicked;

    public void OnPieceClicked(Button sender)
    {
        // Deselected the old button
        ClearSelectedPiece();
        
        // Select the new piece
        selectedButton = sender;
        sender.GetComponent<Image>().color = ActiveButtonColor;
        PieceClicked.Invoke();
    }

    public RepairPiece SelectedRepairPiece
    {
        get
        {
            if (selectedButton != null)
            {
                return selectedButton.GetComponent<WonderGroButton>().HostedPiece;
            }
            return null;
        }
    }

    private Button selectedButton = null;

    public void ClearSelectedPiece()
    {
        if (selectedButton != null)
        {
            selectedButton.GetComponent<Image>().color = AvailableButtonColor;
            selectedButton = null;
        }
    }

    public void ConsumeSelectedPiece()
    {
        if (selectedButton != null)
        {
            ActiveButtons.Remove(selectedButton.gameObject);
            Destroy(selectedButton.gameObject);
            selectedButton = null;
        }
    }

    // Update is called once per frame
    void Update()
    {
        float lastX = 0.0f;
        foreach(GameObject button in ActiveButtons)
        {
            RectTransform rt = button.GetComponent<RectTransform>();

            if (rt.anchoredPosition.x > lastX)
            {
                rt.anchoredPosition += new Vector2(ConveyorVelocity * Time.deltaTime, 0.0f);
            }
            lastX = rt.anchoredPosition.x + rt.rect.xMax + PaddingBetweenButtons;
        }
    }

    public void AddRepairPiece(RepairPiece r)
    {
        GameObject newButton = Instantiate(WonderGroButton, ConveyorParent);
        newButton.GetComponent<WonderGroButton>().HostedPiece = r;
        newButton.GetComponent<RectTransform>().anchoredPosition = new Vector2(ConveyorParent.GetComponent<RectTransform>().rect.width, 0.0f); // Place the button initially at the far right side
        newButton.GetComponent<Image>().color = AvailableButtonColor;
        newButton.GetComponent<Button>().onClick.AddListener(delegate { OnPieceClicked(newButton.GetComponent<Button>()); });
        
        ActiveButtons.Add(newButton);
    }

    public bool ReadyForPiece
    {
        get
        {
            if (ActiveButtons.Count > 0)
            {
                RectTransform rt = ActiveButtons[ActiveButtons.Count - 1].GetComponent<RectTransform>();

                return rt.anchoredPosition.x + rt.rect.xMax < ConveyorParent.GetComponent<RectTransform>().rect.width - PaddingBetweenButtons;
            }

            return true;
        }
    }

}
