﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameState
{
    private const string SAVE_KEY = "Save";

    public static GameState Instance
    {
        get
        {
            if (_instance == null)
            {
                if (PlayerPrefs.HasKey(SAVE_KEY))
                {
                    string json = PlayerPrefs.GetString(SAVE_KEY);
                    _instance = JsonUtility.FromJson<GameState>(json);
                }
                else
                {
                    _instance = new GameState();
                }
            }
            return _instance;
        }
    }

    private static GameState _instance = null;

    public LevelInformation NextLevelToPlay = null;
    public int HighestUnlockedLevel = 1;
    public string PlayerName = "unnamed";
    public int Score = 0;
    public bool MusicEnabled = true;


    public void Save()
    {
        string json = JsonUtility.ToJson(this);
        PlayerPrefs.SetString(SAVE_KEY, json);
        PlayerPrefs.Save();
    }

    public void Reset()
    {
        PlayerPrefs.DeleteKey(SAVE_KEY);
        HighestUnlockedLevel = 1;
        PlayerPrefs.Save();
    }
}
