﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GroPlacer : MonoBehaviour
{
    private ConveyorManager conveyor = null;
    private BoardManager board = null;

    // Start is called before the first frame update
    void Start()
    {
        
    }

    private void Awake()
    {
        conveyor = GetComponent<ConveyorManager>();
        board = GetComponent<BoardManager>();

        for (int i = 0; i < 20; i++)
        {
            GameObject newSelectionObj = Instantiate(SelectionIndicator);
            newSelectionObj.SetActive(false);
            SelectionObjects.Add(newSelectionObj);
        }
    }

    public GameObject SelectionIndicator = null;
    private bool SelectionDirty = true;
    private AxialCoordinate lastMouseCoordinate = AxialCoordinate.zero;

    private List<GameObject> SelectionObjects = new List<GameObject>();

    private int rotationSteps = 0;

    public GameObject WaterTip = null;

    public SoundRandomizer placementSound = null;

    // Update is called once per frame
    void Update()
    {
        // Left mouse clicked
        if (Input.GetMouseButtonDown(0))
        {
            Vector3 mouseWorldPosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
            AxialCoordinate mouseLocation = board.Grid.AxialCoordinateFromWorldPosition(mouseWorldPosition);

            if (board.WithinBounds(mouseLocation))
            {
                WateringCan can = GetComponent<WateringCan>();
                if (can.ChargesAvailable > 0)
                {
                    int activated = 0;
                    foreach (GameObject g in SelectionObjects)
                    {
                        if (!g.activeSelf)
                            break;

                        AxialCoordinate ac = board.Grid.AxialCoordinateFromWorldPosition(g.transform.position);
                        board.FillHole(ac);
                        activated++;
                    }

                    if (activated > 0)
                    {
                        can.UseCharge();

                        // Notify the conveyor that the piece is used up
                        conveyor.ConsumeSelectedPiece();
                        rotationSteps = 0;
                        placementSound.Play();
                    }

                    SelectionDirty = true;
                }
                else
                {
                    // Pulse error message
                    WaterTip.SetActive(true);
                }

            }
        }

        if (WaterTip.activeSelf && GetComponent<WateringCan>().ChargesAvailable > 0)
        {
            WaterTip.SetActive(false);
        }

        if (Input.GetKeyDown(KeyCode.E) || Input.mouseScrollDelta.y < 0 || Input.GetMouseButtonUp(1) || Input.GetMouseButtonDown(2) || Input.GetKeyDown(KeyCode.Space))
        {
            rotationSteps -= 1;
            if (rotationSteps < 0)
                rotationSteps = 5;
            SelectionDirty = true;
        }
        if (Input.GetKeyDown(KeyCode.Q) || Input.mouseScrollDelta.y > 0)
        {
            rotationSteps += 1;
            rotationSteps = (rotationSteps % 6);
            SelectionDirty = true;
        }


        if (conveyor.SelectedRepairPiece != null)
        {
            Vector3 averageWorldOffset = Vector3.zero;
            foreach(AxialCoordinate ac in conveyor.SelectedRepairPiece.Pieces)
            {
                averageWorldOffset += board.Grid.WorldPositionFromAxialPosition(new AxialCoordinate(new CubeCoordinate(ac).Rotate(rotationSteps)));
            }
            averageWorldOffset /= conveyor.SelectedRepairPiece.Pieces.Count;


            Vector3 mouseWorldPosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
            AxialCoordinate mouseLocation = board.Grid.AxialCoordinateFromWorldPosition(mouseWorldPosition - averageWorldOffset);

            if (!lastMouseCoordinate.Equals(mouseLocation))
            {
                SelectionDirty = true;
                lastMouseCoordinate = mouseLocation;
            }

        }

        if (SelectionDirty)
        {
            foreach(GameObject o in SelectionObjects)
            {
                o.SetActive(false);
            }

            RepairPiece piece = conveyor.SelectedRepairPiece;
            if (piece != null)
            {
                for (int i = 0; i < piece.Pieces.Count; i++)
                {
                    GameObject selectionObject = SelectionObjects[i];
                    selectionObject.SetActive(true);

                    AxialCoordinate pieceLoc = piece.Pieces[i];
                    pieceLoc = new AxialCoordinate(new CubeCoordinate(pieceLoc).Rotate(rotationSteps));

                    Vector3 selectPiecePosition = board.Grid.WorldPositionFromAxialPosition(lastMouseCoordinate + pieceLoc);
                    selectPiecePosition += new Vector3(0.0f, 2.0f, 0.0f);
                    selectionObject.transform.position = selectPiecePosition;
                }
            }

            SelectionDirty = false;

        }
    }
}
