﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LevelDirector : MonoBehaviour
{
    // Start is called before the first frame update
    void Start()
    {
        if (GameState.Instance.NextLevelToPlay != null)
        {
            HoleSizes = new MarbleBag<int>(GameState.Instance.NextLevelToPlay.HoleSizes);
            RepairPieces = new MarbleBag<RepairPiece>(GameState.Instance.NextLevelToPlay.RepairPieces);
            HoleCreationIntervals = GameState.Instance.NextLevelToPlay.HoleCreationIntervals;
        }
        else
        {
            HoleSizes = new MarbleBag<int>(DefaultHoleSizes);
            RepairPieces = new MarbleBag<RepairPiece>(DefaultRepairPieces);
            HoleCreationIntervals = DefaultHoleCreationIntervals;
        }

        HoleCreationIndex = 0;
        NextHoleTime = Time.time + NextHoleCreationInterval;
        NextRepairPieceTime = Time.time + RepairPieceInterval;

    }

    public float NextHoleTime = 0.0f;
    private int HoleCreationIndex = 0;
    private int NextHoleCreationInterval
    {
        get
        {
            if (HoleCreationIndex >= HoleCreationIntervals.Length)
                HoleCreationIndex = 0;

            int retval = HoleCreationIntervals[HoleCreationIndex];
            HoleCreationIndex++;

            return retval;
        }
    }
    public int[] HoleCreationIntervals = null;
    public int[] DefaultHoleCreationIntervals = { 5, 5, 0, 0, 15, 5 };

    public float NextRepairPieceTime = 0.0f;
    public float RepairPieceInterval = 5.0f;

    public RepairPiece[] DefaultRepairPieces = null;
    private MarbleBag<RepairPiece> RepairPieces;

    public int[] DefaultHoleSizes = { 3, 4, 5, 5, 6, 6, 6, 7, 7, 8, 9 };
    private MarbleBag<int> HoleSizes;

    // Update is called once per frame
    void Update()
    {
        if (Time.time > NextHoleTime)
        {
            // Time to make a hole
            BoardManager bm = GetComponent<BoardManager>();
            NextHoleTime += NextHoleCreationInterval;

            List<AxialCoordinate> candidateLocations = new List<AxialCoordinate>();
            // First try to find a starting root location that has no hole. Try a maximum of 10 times.
            for (int i = 0; i < 10; i ++)
            {
                AxialCoordinate holeStartingPoint = new AxialCoordinate( new OffsetCoordinate(
                Random.Range((int)-bm.halfWidth, bm.halfWidth + 1),
                Random.Range((int)-bm.halfHeight, bm.halfHeight + 1)
                ));

                if (bm.WithinBounds(holeStartingPoint) && !bm.IsHoleAt(holeStartingPoint))
                {
                    // Add the location to the list
                    candidateLocations.Add(holeStartingPoint);
                    break;
                }
            }

            int holesLeftToCreate = HoleSizes.Next;
            while (candidateLocations.Count > 0 && holesLeftToCreate > 0)
            {
                // Choose a location in the list
                int chosenindex = Random.Range(0, candidateLocations.Count);
                AxialCoordinate createHoleAt = candidateLocations[chosenindex];
                candidateLocations.RemoveAll(loc => loc.Equals(createHoleAt));

                // Create the hole at the selected location
                bm.CreateOrGetHole(createHoleAt);
                holesLeftToCreate -= 1;

                // Add all non-hole neighbours to the candidate list
                foreach(AxialCoordinate direction in AxialCoordinate.Directions)
                {
                    AxialCoordinate neighbour = createHoleAt + direction;
                    if (bm.WithinBounds(neighbour) && !bm.IsHoleAt(neighbour))
                        candidateLocations.Add(neighbour);
                }
            }
        }

        if (Time.time > NextRepairPieceTime)
        {
            // Time to create a repair piece
            NextRepairPieceTime += RepairPieceInterval;

            if (GetComponent<ConveyorManager>().ReadyForPiece)
                GetComponent<ConveyorManager>().AddRepairPiece(RepairPieces.Next);
        }
    }
}
