﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
#if UNITY_EDITOR
using UnityEditor;
#endif

#if UNITY_EDITOR
[CustomEditor(typeof(PieceEditor))]
public class PieceEditorHelpers : Editor
{
    public override void OnInspectorGUI()
    {
        base.OnInspectorGUI();

        PieceEditor pe = (PieceEditor)target;
        if (GUILayout.Button("Refresh"))
        {
            pe.Refresh();
        }
    }
}
#endif

public class PieceEditor : MonoBehaviour
{

    public RepairPiece EditedPiece = null;
    public GameObject HexObject = null;

    HexGrid EditGrid = new HexGrid();


    // Start is called before the first frame update
    void Start()
    {
        EditGrid.SetGridSize(0.5f);
        RenderPiece();
    }

    public void Refresh()
    {
        Rose[] roses = FindObjectsOfType<Rose>();
        foreach (Rose r in roses)
        {
            Destroy(r.gameObject);
        }
        RenderPiece();

    }

    public void RenderPiece()
    {
        foreach(AxialCoordinate ac in EditedPiece.Pieces)
        {
            CreatePieceAt(ac);
        }
    }

    public void CreatePieceAt(AxialCoordinate ac)
    {
        GameObject.Instantiate(HexObject, EditGrid.WorldPositionFromAxialPosition(ac), Quaternion.identity);
    }

    public void DeletePieceAt(AxialCoordinate ac)
    {
        EditGrid.QueryForCollidersAtAxialPosition(ac, 1);
        foreach (Collider c in EditGrid.IterateQueriedColliders())
        {
            Destroy(c.gameObject);
        }
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetMouseButtonDown(0))
        {
            Vector3 mouseWorldPosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
            AxialCoordinate holeLocation = EditGrid.AxialCoordinateFromWorldPosition(mouseWorldPosition);

            if (EditedPiece.Pieces.Contains(holeLocation))
            {
                EditedPiece.Pieces.Remove(holeLocation);
                DeletePieceAt(holeLocation);
            }
            else
            {
                EditedPiece.Pieces.Add(holeLocation);
                CreatePieceAt(holeLocation);
            }
#if UNITY_EDITOR
            EditorUtility.SetDirty(EditedPiece);
#endif
        }
    }

}
